(* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
This is the Renapp.m file, a Mathematica routine for FeynRules where we define everything related
to the renormalization in the FeynRules environment.

Created by: Duarte Fontes
Email: duarte.fontes@kit.edu
Last Update: 2/April/2025
* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *)

combetas={};
ancCTve2tot={};

If[ToString[LCTGauge//InputForm]!="0",LCTGauge=GetCT[LEIGauge],LCTGauge=0];
If[ToString[LCTHiggs//InputForm]!="0",LCTHiggs=GetCT[LEIHiggs],LCTHiggs=0];
If[ToString[LCTGhost//InputForm]!="0",LCTGhost=GetCT[LEIGhost],LCTGhost=0];
If[ToString[LCTGF//InputForm]!="0",LCTGF=GetCT[LEIGF],LCTGF=0];
If[ToString[LCTFermions//InputForm]!="0",LCTFermions=GetCT[LEIFermions],LCTFermions=0];
If[ToString[LCTYukawa//InputForm]!="0",LCTYukawa=GetCT[LEIYukawa],LCTYukawa=0];

LoCTfin = LCTGauge + LCTHiggs + LCTGhost + LCTGF + LCTFermions + LCTYukawa;

If[DeBug, str7 = OpenWrite["MyProgressPos.m"]];
MDB[str7];
(* Write the final list of counterterms: *)
CTppplist = {};
Do[CTppplist = Append[CTppplist,(((Coefficient[LoCTfin, combetas[[i]]] /.cleanetas)* combetas[[i]]) /.gamrep) /.pprops /. assodots // InputForm], {i, 1, Length[combetas]}];
CTinilist=CTppplist /. MyRestr /. unitetas;
Clear[i];
MDB[str7];

(* Some details to include in the external file where CTinilist is to be written: *)
VList := ancCTve2tot
VoutL = {};
Do[
		auxVout = {};
		Do[auxVout = Append[auxVout,VList[[i,1,j,1]]],{j,1,Length[VList[[i,1]]]}];
		VoutL = Append[VoutL,auxVout];
		VoutL = VoutL,{i,1,Length[VList]}];
VoutL2 = {};
Do[VoutL2 = Append[VoutL2, {}], {i, 1, Length[VoutL]}];
Do[Do[VoutL2[[i]] = Append[VoutL2[[i]], ToString[InputForm[VoutL[[i, j]]]]], {j, 1, Length[VoutL[[i]]]}], {i, 1, Length[VoutL]}];
AoutL = {};
Do[If[ToString[ToExpression[palist2[[i, 2]]<>"bar"]/.antimap]==palist2[[i, 2]],
AoutL = Append[AoutL, palist3[[i,2]] -> ToExpression[palist2[[i,2]]<>"bar"]];
AoutL = Append[AoutL, ToExpression[palist2[[i,2]]<>"bar"] -> palist3[[i,2]]]],{i,1,Length[palist3]}];
MDB[str7];
CTpreord1 = combetas /. comeback;
CTpreord2 = {};
Do[ordaux = {};
  		If[Length[CTpreord1[[i]]] == 0,
   				ordaux = Append[ordaux, ToString[CTpreord1[[i]]]], 
   				If[Length[CTpreord1[[i]]] == 2 && IntegerQ[CTpreord1[[i, 2]]], 
    				Do[ordaux = Append[ordaux, ToString[CTpreord1[[i, 1]]]], {m, 1, CTpreord1[[i, 2]]}], 
    				Do[If[Length[CTpreord1[[i, j]]] > 0,
      							Do[ordaux = Append[ordaux, ToString[CTpreord1[[i, j, 1]]]], {k, 1, CTpreord1[[i, j, 2]]}],
      							ordaux = Append[ordaux, ToString[CTpreord1[[i, j]]]]], {j, 1, Length[CTpreord1[[i]]]}]]];
  		CTpreord2 = Append[CTpreord2, ordaux], {i, 1, Length[CTpreord1]}];
CTord = CTpreord2;
Do[Do[If[Sort[CTord[[i]]] === Sort[VoutL2[[j]]],
					CTord[[i]] = VoutL2[[j]]], {j, 1, Length[VoutL2]}], {i, 1, Length[CTord]}];
MDB[str7];

(* Finally, the external file to the FeynCalc envirnonment: *)
str = OpenWrite["CTpre.m"];
Do[WriteString[str, "+++++ "<>ToString[CTinilist[[i]]]<>"\n"],{i,1,Length[CTinilist]}];
WriteString[str, "\n"];
WriteString[str, "CTord := "];
WriteString[str, ToString[CTord]];
WriteString[str, "\n"];
WriteString[str, "cleanetas := "];
WriteString[str, ToString[cleanetas]];
WriteString[str, "\n"];
WriteString[str, "VertLi := "];
WriteString[str, ToString[VoutL]];
WriteString[str, "\n"];
WriteString[str, "AntiRep := "];
WriteString[str, ToString[AoutL]];
Close[str];
MDB[str7];

(* Now, we focus on the printing of the Feynman rules for the counterterms. *)
(* We start by creating some (more) lists, which will be very useful for the printing *)
palistbri1 = {};
Do[palistbri1 = Append[palistbri1, {palist3[[i, 2]]}];
  If[ToString[StandardForm[ToExpression[ToString[palist3[[i, 2]]] <> "bar"]]] != ToString[InputForm[ToExpression[ToString[palist3[[i, 2]]] <> "bar"]]], 
   palistbri1 = Append[palistbri1, {ToExpression[ToString[palist3[[i, 2]]] <> "bar"]}]], {i, 1, Length[palist3]}];
palistbri2 = {};
Do[palistbri2 = Append[palistbri2, {ToString[InputForm[palistbri1[[i, 1]]]]}], {i, 1, Length[palistbri1]}];
Do[If[StringLength[palistbri2[[i, 1]]] > 2 && StringTake[palistbri2[[i, 1]], -3] == "bar", 
   	   Do[If[StringDelete[palistbri2[[i, 1]], "bar"] == ToString[InputForm[palist3[[k, 2]]]], 
     				palistbri2[[i]] = Append[palistbri2[[i]], ToString[InputForm[palist3[[k, 1]]]]]; 
     				Break[]], {k, 1, Length[palist3]}], 
   		Do[If[palistbri2[[i, 1]] == ToString[InputForm[palist3[[k, 2]]]], 
                   palistbri2[[i]] = Append[palistbri2[[i]], ToString[InputForm[palist3[[k, 1]]]]]; 
     			   Break[]], {k, 1, Length[palist3]}]], {i, 1, Length[palistbri2]}];
palistbri3 = palistbri2 // ToExpression;
palistbri4 = palistbri3;
palistbri5 = {};
Do[palistbri5 = Append[palistbri5, {}], {i, 1, Length[palistbri4]}];
Do[palistbri5[[i]] = Append[palistbri5[[i]], ToString[palistbri4[[i, 1]]]];
       palistbri5[[i]] = Append[palistbri5[[i]], ToString[palistbri4[[i, 2]]]], {i, 1, Length[palistbri4]}];
MDB[str7];
CTbri1 = {};
Do[CTbri1 = Append[CTbri1, {}], {i, 1, Length[CTord]}];
Do[Do[Do[If[CTord[[i, j]] == palistbri5[[k, 1]], CTbri1[[i]] = Append[CTbri1[[i]], palistbri5[[k, 2]]]], {k, 1, Length[palistbri5]}], {j, 1, Length[CTord[[i]]]}], {i, 1, Length[CTord]}];
CTbri2 = {};
Do[CTbri2 = Append[CTbri2, {}], {i, 1, Length[CTbri1]}];
Do[Do[If[! MemberQ[CTbri2[[i]], CTbri1[[i, j]]], CTbri2[[i]] = Append[CTbri2[[i]], CTbri1[[i, j]]]], {j, 1, Length[CTbri1[[i]]]}], {i, 1, Length[CTbri1]}];
MDB[str7];
CTmidlist1 = {};
Do[CTmidlist1 = Append[CTmidlist1, ToString[CTinilist[[i]]]], {i, 1, Length[CTinilist]}];
CTmidlist2 = CTmidlist1 // ToExpression;
CTmidlist3 = (CTmidlist2 /. unitetas) /. MyRestr /. MyRestr;
CTmidlist4 = {};
Do[CTmidlist4 = Append[CTmidlist4, {ToString[InputForm[CTmidlist3[[i]]]]}], {i, 1, Length[CTmidlist3]}];
MDB[str7];

(* Now comes the printing part itself. We divide it by sections *)

PrePrintCTTads = {};
Do[If[Length[CTord[[i]]] == 1,
            PrePrintCTTads = Append[PrePrintCTTads, {CTord[[i]],CTmidlist4[[i]]}]], {i, 1, Length[CTord]}];
If[PrePrintCTTads != {},
  	str1 = OpenWrite["PrePrintCTTads.m"];
    Do[WriteString[str1, "{{{{{{", PrePrintCTTads[[i, 1, 1]], "}}}}}},{{{{{{ ", 
    		PrePrintCTTads[[i, 2, 1]], " }}}}}}\n \n"], {i, 1, Length[PrePrintCTTads]}];
  	Close[str1]];

PrePrintCTProps = {};
Do[If[Length[CTord[[i]]] == 2,
            AppendTo[PrePrintCTProps, {CTord[[i]],CTmidlist4[[i]]}]], {i, 1, Length[CTord]}];
If[PrePrintCTProps != {},
  	str1 = OpenWrite["PrePrintCTProps.m"];
    Do[WriteString[str1, "{{{{{{", PrePrintCTProps[[i, 1, 1]], ",", PrePrintCTProps[[i, 1, 2]], "}}}}}},{{{{{{ ", 
    		PrePrintCTProps[[i, 2, 1]], " }}}}}}\n \n"], {i, 1, Length[PrePrintCTProps]}];
  	Close[str1]];

PrePrintCTRulesGauge = {};
Do[If[Length[CTord[[i]]] > 2 && CTbri2[[i]] == {"V"},
      AppendTo[PrePrintCTRulesGauge, {CTord[[i]],CTmidlist4[[i]]}]], {i, 1, Length[CTord]}];
If[PrePrintCTRulesGauge != {},
    str1 = OpenWrite["PrePrintCTRulesGauge.m"];
    Do[ auxstr="";
        Do[auxstr=auxstr<>PrePrintCTRulesGauge[[i, 1, j]]<>",", {j, 1, Length[PrePrintCTRulesGauge[[i,1]]]}];
        auxstr=StringDrop[auxstr,-1];
        WriteString[str1, "{{{{{{"<>auxstr<>"}}}}}},{{{{{{ "<>PrePrintCTRulesGauge[[i, 2, 1]]<>" }}}}}}\n \n"], {i, 1, Length[PrePrintCTRulesGauge]}];
    Close[str1]];

PrePrintCTRulesHiggs = {};
Do[If[Length[CTord[[i]]] > 2 && MemberQ[CTbri2[[i]], "S"] && ! MemberQ[CTbri2[[i]], "F"] && ! MemberQ[CTbri2[[i]], "U"],
      AppendTo[PrePrintCTRulesHiggs, {CTord[[i]],CTmidlist4[[i]]}]], {i, 1, Length[CTord]}];
If[PrePrintCTRulesHiggs != {},
    str1 = OpenWrite["PrePrintCTRulesHiggs.m"];
    Do[ auxstr="";
        Do[auxstr=auxstr<>PrePrintCTRulesHiggs[[i, 1, j]]<>",", {j, 1, Length[PrePrintCTRulesHiggs[[i,1]]]}];
        auxstr=StringDrop[auxstr,-1];
        WriteString[str1, "{{{{{{"<>auxstr<>"}}}}}},{{{{{{ "<>PrePrintCTRulesHiggs[[i, 2, 1]]<>" }}}}}}\n \n"], {i, 1, Length[PrePrintCTRulesHiggs]}];
    Close[str1]];

PrePrintCTRulesFermions = {};
Do[If[Length[CTord[[i]]] > 2 && MemberQ[CTbri2[[i]], "F"] && ! MemberQ[CTbri2[[i]], "S"],
      AppendTo[PrePrintCTRulesFermions, {CTord[[i]],CTmidlist4[[i]]}]], {i, 1, Length[CTord]}];
If[PrePrintCTRulesFermions != {},
    str1 = OpenWrite["PrePrintCTRulesFermions.m"];
    Do[ auxstr="";
        Do[auxstr=auxstr<>PrePrintCTRulesFermions[[i, 1, j]]<>",", {j, 1, Length[PrePrintCTRulesFermions[[i,1]]]}];
        auxstr=StringDrop[auxstr,-1];
        WriteString[str1, "{{{{{{"<>auxstr<>"}}}}}},{{{{{{ "<>PrePrintCTRulesFermions[[i, 2, 1]]<>" }}}}}}\n \n"], {i, 1, Length[PrePrintCTRulesFermions]}];
    Close[str1]];

PrePrintCTRulesYukawa = {};
Do[If[Length[CTord[[i]]] > 2 && MemberQ[CTbri2[[i]], "F"] && MemberQ[CTbri2[[i]], "S"],
      AppendTo[PrePrintCTRulesYukawa, {CTord[[i]],CTmidlist4[[i]]}]], {i, 1, Length[CTord]}];
If[PrePrintCTRulesYukawa != {},
    str1 = OpenWrite["PrePrintCTRulesYukawa.m"];
    Do[ auxstr="";
        Do[auxstr=auxstr<>PrePrintCTRulesYukawa[[i, 1, j]]<>",", {j, 1, Length[PrePrintCTRulesYukawa[[i,1]]]}];
        auxstr=StringDrop[auxstr,-1];
        WriteString[str1, "{{{{{{"<>auxstr<>"}}}}}},{{{{{{ "<>PrePrintCTRulesYukawa[[i, 2, 1]]<>" }}}}}}\n \n"], {i, 1, Length[PrePrintCTRulesYukawa]}];
    Close[str1]];

PrePrintCTRulesGhosts = {};
Do[If[Length[CTord[[i]]] > 2 && MemberQ[CTbri2[[i]], "U"],
      AppendTo[PrePrintCTRulesGhosts, {CTord[[i]],CTmidlist4[[i]]}]], {i, 1, Length[CTord]}];
If[PrePrintCTRulesGhosts != {},
    str1 = OpenWrite["PrePrintCTRulesGhosts.m"];
    Do[ auxstr="";
        Do[auxstr=auxstr<>PrePrintCTRulesGhosts[[i, 1, j]]<>",", {j, 1, Length[PrePrintCTRulesGhosts[[i,1]]]}];
        auxstr=StringDrop[auxstr,-1];
        WriteString[str1, "{{{{{{"<>auxstr<>"}}}}}},{{{{{{ "<>PrePrintCTRulesGhosts[[i, 2, 1]]<>" }}}}}}\n \n"], {i, 1, Length[PrePrintCTRulesGhosts]}];
    Close[str1]];

If[DeBug,Close[str7]];

(* Protecting variables in use: *)
If[RenoLogic,
ClearAll[variableQ];
variableQ[name_String] := MatchQ[ToExpression[name, InputForm, OwnValues], 
                    Except[{} | {_ :> (Function | CompiledFunction)[___]}]];
ToProtect = Select[Names["Global`*"], variableQ];
Do[ToString[StringForm[ToProtect[[i]] <> "//Protect"]] // ToExpression,{i,1,Length[ToProtect]}]];
